# include <QtGui>

# include "bildershowgui.h"

BildershowGui::BildershowGui(QWidget *parent): QDialog(parent)
{
	setupUi(this);

	QTextCodec::setCodecForTr(QTextCodec::codecForName("UTF-8"));

	QTimer::singleShot(0, this, SLOT(converttest()));

// Standardfarben für Hintergrund, Titel und Links setzen
	QColor farbe1 = Qt::black;
	hintergrundfarbeLineEdit->setText(farbe1.name());
	bcolorLineEdit->setStyleSheet("background-color: "+farbe1.name()+";");
	QColor farbe2 = Qt::white;
	titelfarbeLineEdit->setText(farbe2.name());
	tcolorLineEdit->setStyleSheet("background-color: "+farbe2.name()+";");
	QColor farbe3;
	farbe3.setNamedColor("#00CCFF");
	linkfarbeLineEdit->setText(farbe3.name());
	lcolorLineEdit->setStyleSheet("background-color: "+farbe3.name()+";");

// Eingabe auf Hexadezimale Zeichen beschränken
	QRegExp regExp("[#][A-Fa-f0-9][A-Fa-f0-9][A-Fa-f0-9][A-Fa-f0-9][A-Fa-f0-9][A-Fa-f0-9]");
	hintergrundfarbeLineEdit->setValidator(new QRegExpValidator(regExp, this));
	titelfarbeLineEdit->setValidator(new QRegExpValidator(regExp, this));
	linkfarbeLineEdit->setValidator(new QRegExpValidator(regExp, this));



	connect(thumbsRightSpinBox, SIGNAL(valueChanged(int)), this, SLOT(thumbssize(int)));

	connect(hintergrundfarbeButton, SIGNAL(clicked()), this, SLOT(chooseBColor()));
	connect(titelfarbeButton, SIGNAL(clicked()), this, SLOT(chooseTColor()));
	connect(linkfarbeButton, SIGNAL(clicked()), this, SLOT(chooseLColor()));

	connect(hintergrundfarbeLineEdit, SIGNAL(editingFinished()), this, SLOT(updateBColor()));
	connect(titelfarbeLineEdit, SIGNAL(editingFinished()), this, SLOT(updateTColor()));
	connect(linkfarbeLineEdit, SIGNAL(editingFinished()), this, SLOT(updateLColor()));

	connect(openButton, SIGNAL(clicked()), this, SLOT(openDialog()));
	connect(createButton, SIGNAL(clicked()), this, SLOT(create()));

	connect(&proc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(convertBeendet(int, QProcess::ExitStatus)));

}


// ----------------------End of Main --------------------------------

/****************************************
 Funktion Zahlen korrekt runden
*****************************************/

int rundung(double eingzahl)
{
	double kommazahl;
	int hauptzahl,gerundet;

	hauptzahl=(int)eingzahl;
	kommazahl=eingzahl-(double)hauptzahl;
	if(kommazahl<0.5)
		{
		gerundet=hauptzahl;
		}
	else
		{
		gerundet=hauptzahl+1;
		}
	return gerundet;
}


/******************************************************************
 öffnen Dialog und eintragen der Bilderliste in das QTableWidget
*******************************************************************/

void BildershowGui::openDialog()
{
	QString initialDir = openLineEdit->text();
	if(initialDir.isEmpty())
		initialDir = QDir::homePath();

	QString openDir = QFileDialog::getExistingDirectory(
		this,
		tr("Verzeichnis öffnen"),
		initialDir,
		QFileDialog::DontConfirmOverwrite);

	if(!openDir.isEmpty())
		{
		if(!openDir.endsWith("/"))
			openDir = openDir+"/";

		openLineEdit->setText(openDir);

		int vaufl,haufl;
		QString vl,hl,aufl;

		QStringList namefilters;
		namefilters << "*.jpeg" << "*.jpg";
		QDir files(openDir);
		files.setNameFilters(namefilters);
		files.setSorting(QDir::Name);

		QFileInfoList jpgliste=files.entryInfoList();

		dateiTableWidget->clear();
		dateiTableWidget->setRowCount(jpgliste.size());
		dateiTableWidget->setColumnCount(3);
		dateiTableWidget->horizontalHeader()->resizeSection(0,300);
		dateiTableWidget->horizontalHeader()->resizeSection(1,140);
		dateiTableWidget->horizontalHeader()->resizeSection(2,70);
		dateiTableWidget->setHorizontalHeaderItem(0, new QTableWidgetItem(tr("Bilder")));
		dateiTableWidget->setHorizontalHeaderItem(1, new QTableWidgetItem(tr("Auflösung")));
		dateiTableWidget->setHorizontalHeaderItem(2, new QTableWidgetItem(tr("Info")));

		for (int i=0;i<jpgliste.size();i++)
			{
			QFileInfo fileInfo = jpgliste.at(i);
			QString name = fileInfo.fileName();
			QString filepath=fileInfo.absoluteFilePath(); // absoluter Pfad+Filename

			QImageReader bild(filepath);
			QSize img=bild.size();
			vaufl=img.width();
			haufl=img.height();
			vl=vl.setNum(vaufl);
			hl=hl.setNum(haufl);
			aufl=vl+"x"+hl;

			dateiTableWidget->setItem(i,0, new QTableWidgetItem(name));
			dateiTableWidget->setItem(i,1, new QTableWidgetItem(aufl));
			}
		createButton->setEnabled(true);
		}
}



/***********************************************************
 Create - thumbnails und Bilder erzeugen
************************************************************/

void BildershowGui::create()
{
	int error;

	openButton->setEnabled(false);
	createButton->setEnabled(false);

// Daten holen
	anz=dateiTableWidget->rowCount();
	verzeichnis=openLineEdit->text();
	int border=borderSpinBox->value();
	int thtitel,thloesung;
	int vauflthumbs=thumbsLeftSpinBox->value();
	hauflthumbs=thumbsRightSpinBox->value();
	int padding=paddingSpinBox->value();
	int spalten=spaltenSpinBox->value();
	QString hfarbe=hintergrundfarbeLineEdit->text();
	QString tfarbe=titelfarbeLineEdit->text();
	QString lfarbe=linkfarbeLineEdit->text();
	if(bildnameCheckBox->isChecked())
		thtitel=1;
	else
		thtitel=0;

	if(aufloesungCheckBox->isChecked())
		thloesung=1;
	else
		thloesung=0;

// Titel holen und Abfrage ob Leer
	QString titel=titelLineEdit->text();
	if(titel.isEmpty())
		{
		font.setPointSize(12);
		QMessageBox msgBox;
		msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
		msgBox.setDefaultButton(QMessageBox::Yes);
		msgBox.setFont(font);
		msgBox.setIcon(QMessageBox::Information);
		msgBox.setWindowTitle("Bildershow-gui");
		msgBox.setText(tr("Achtung! Sie haben keinen Titel angegeben!<br>"
			"Möchten Sie die Erstellung ohne Titel fortsetzen?"));
		switch (msgBox.exec())
			{
			case QMessageBox::Yes:
				break;
			case QMessageBox::No:
				openButton->setEnabled(true);
				createButton->setEnabled(true);
				return;
			}
		}

// Spalte 3 (Info) leeren

	for(int x=0;x<anz;x++)
		{
		dateiTableWidget->setItem(x,2, new QTableWidgetItem(" "));
		}

// Ordner anlegen, bzw. Inhalt löschen
	QStringList namefilters;
	namefilters << "*.html" << "*.jpg";
	QString verzOriginal;
	verzOriginal=verzeichnis+"original/";
	QDir createDir(verzeichnis);
	QDir testDir(verzOriginal);
	testDir.setNameFilters(namefilters);
	if(testDir.exists())
		{
		QFileInfoList dateiliste=testDir.entryInfoList();
		foreach (QFileInfo delfile, dateiliste)
			{
			testDir.remove(delfile.fileName());
			}
		error=1;
		}
	else	{
		error=createDir.mkdir("original");
		}

	if(error==0)
		{
		errorDir();
		openButton->setEnabled(true);
		createButton->setEnabled(true);
		return;
		}

	verzThumbs=verzeichnis+"thumbs/";
	testDir.setPath(verzThumbs);
	if(testDir.exists())
		{
		QFileInfoList dateiliste=testDir.entryInfoList();
		foreach (QFileInfo delfile, dateiliste)
			{
			testDir.remove(delfile.fileName());
			}
		error=1;
		}
	else	{
		error=createDir.mkdir("thumbs");
		}

	if(error==0)
		{
		errorDir();
		openButton->setEnabled(true);
		createButton->setEnabled(true);
		return;
		}

// Dateinamen holen für die Weiterverarbeitung

	picname.clear();
	htmname.clear();

	QString vert,hori;
	QList<double> vaufl;
	QList<double> haufl;
	QList<double> coeff;
	QString tmp;

	QString SourceFile = verzeichnis+dateiTableWidget->item(anz-1,0)->text();

	picname << dateiTableWidget->item(anz-1,0)->text();
	htmname << QFileInfo(SourceFile).completeBaseName()+".html";
	tmp = dateiTableWidget->item(anz-1,1)->text();
	vert = tmp.mid(0,tmp.indexOf("x"));
	hori = tmp.mid(tmp.indexOf("x")+1);
	vaufl << vert.toDouble();
	haufl << hori.toDouble();
	coeff << vaufl[0]/haufl[0];

	for(int y=0;y<anz;y++)
		{
		SourceFile = verzeichnis+dateiTableWidget->item(y,0)->text();
		picname << dateiTableWidget->item(y,0)->text();
		htmname << QFileInfo(SourceFile).completeBaseName()+".html";
		tmp = dateiTableWidget->item(y,1)->text();
		vert = tmp.mid(0,tmp.indexOf("x"));
		hori = tmp.mid(tmp.indexOf("x")+1);
		vaufl << vert.toDouble();
		haufl << hori.toDouble();
		coeff << vaufl[y+1]/haufl[y+1];
		}
	picname << picname[1];
	htmname << htmname[1];
	vaufl << vaufl[1];
	haufl << haufl[1];
	coeff << coeff[1];

// HTML-Seiten für die eingebetteten Original-Bilder erstellen

	QApplication::setOverrideCursor(Qt::WaitCursor);

	for(int i=1;i<=anz;i++)
		{
		QString overz=verzOriginal+htmname[i];
		QFile ofiles(overz);
		if(ofiles.open(QFile::WriteOnly | QFile::Truncate))
			{
			QTextStream origfiles(&ofiles);

			origfiles << "<html>" << endl;
			origfiles << "<head>" << endl;
			origfiles << "<title>" << titel << "-" << picname[i] << "</title>" << endl;
			origfiles << "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">" << endl;
			origfiles << "</head>" << endl;
			origfiles << endl;
			origfiles << "<body bgcolor=\"" << hfarbe << "\" text=\"" << lfarbe << "\" link=\"" << lfarbe << "\" vlink=\"" << lfarbe << "\" alink=\"" << lfarbe << "\">" << endl;
			origfiles << "<p align=\"center\">" << endl;
			origfiles << "<a href=\"" << htmname[i-1] << "\">Vorheriges</a> |" << endl;
			origfiles << "<a href=\"" << htmname[(i+1)] << tr("\">Nächstes</a> |") << endl;
			origfiles << "<a href=\"../thumbs.html\">Miniaturansicht</a>" << endl;
			origfiles << "</p>" << endl;
			origfiles << "<table border=\"0\" cellpadding=\"2\" cellspacing=\"5\" align=\"center\">" << endl;
			origfiles << "<tr>" << endl;
			origfiles << "<td style=\"border: " << border << "px solid " << tfarbe << ";\"><img src=\"../" << picname[i] << "\" alt=\"" << picname[i] << "\" width=\"" << vaufl[i] << "\" height=\"" << haufl[i] << "\"></td>" << endl;
			origfiles << "</tr>" << endl;
			origfiles << "<tr>" << endl;
			origfiles << "<td align=\"center\">" << picname[i] << "</td>" << endl;
			origfiles << "</tr>" << endl;
			origfiles << "</table>" << endl;
			origfiles << "</body>" << endl;
			origfiles << "</html>" << endl;
			}
		ofiles.close();
		}

// thumbs.html erzeugen

	int c=1;
	int b;

	QString thumbsverz=verzeichnis+"thumbs.html";
	QFile thhtml(thumbsverz);
	if(thhtml.open(QFile::WriteOnly | QFile::Truncate))
		{
		QTextStream thumbhtml(&thhtml);

		thumbhtml << "<html>" << endl;
		thumbhtml << "<head>" << endl;
		thumbhtml << "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">" << endl;
		thumbhtml << "<title>" << titel << "</title>" << endl;
		thumbhtml << "</head>" << endl;
		thumbhtml << endl;
		thumbhtml << "<body bgcolor=\"" << hfarbe << "\" text=\"" << tfarbe << "\" link=\"" << tfarbe << "\" vlink=\"" << tfarbe << "\" alink=\"" << tfarbe << "\">" << endl;
		thumbhtml << "<table border=\"" << border << "\" cellpadding=\"" << padding << "\" cellspacing=\"0\" align=\"center\">" << endl;
		thumbhtml << "<tr>" << endl;
		thumbhtml << "<td valign=\"middle\" align=\"center\" colspan=\"" << spalten << "\" height=\"100\"><h1>" << titel << "</h1></td>" << endl;
		thumbhtml << "</tr>" << endl;
		thumbhtml << "<tr>" << endl;
		b=1;
		while(c<=anz && b<=spalten)
			{
			thumbhtml << "<td width=\"" << vauflthumbs << "\" align=\"center\">" << endl;
			thumbhtml << "<a href=\"original/" << htmname[c] << "\"><img src=\"thumbs/" << picname[c] << "\"  alt=\"" << picname[c] << "\" width=\"" << (rundung(hauflthumbs*coeff[c])) << "\" height=\"" << hauflthumbs << "\" border=\"0\"></a>" << endl;
			if(thtitel==1)
				{
				thumbhtml << "<div>" << picname[c] << "</div>" << endl;
				}
			else
				{
				thumbhtml << "<div>&nbsp;</div>" << endl;
				}
			if(thloesung==1)
				{
				thumbhtml << "<div>" << vaufl[c] << "x" << haufl[c] << "</div>" << endl;
				}
			thumbhtml << "</td>" << endl;
			b=b+1;
			c=c+1;
			if(b>spalten)
				{
				thumbhtml << "</tr>" << endl;
				thumbhtml << "<tr>" << endl;
				b=1;
				}
			}
		if(b!=1)
			{
			thumbhtml << "<td valign=\"middle\" align=\"center\" colspan=\"" << ((spalten+1)-b) << "\" height=\"100\">&nbsp;</td>" << endl;
			}
		thumbhtml << "</tr>" << endl;
		thumbhtml << "<tr>" << endl;
		thumbhtml << "<td valign=\"middle\" align=\"center\" colspan=\"" << spalten << tr("\" height=\"80\"><font face=\"Arial, Helvetica, sans-serif\">Zurück</td>") << endl;
		thumbhtml << "</tr>" << endl;
		thumbhtml << "</table>" << endl;
		thumbhtml << "</body>" << endl;
		thumbhtml << "</html>" << endl;

		}
	thhtml.close();

// thumbs erzeugen

	z=1;
	procThumbs();



}






/***********************************************************
 3x Farbe auswählen für Hintergrund-, Titel und Linkfarbe
************************************************************/

void BildershowGui::chooseBColor()
{
	QColor initialcolor;
	QString farbe;

	initialcolor.setNamedColor(hintergrundfarbeLineEdit->text());

	QColor color = QColorDialog::getColor(initialcolor, this);
	if (color.isValid())
		{
		farbe="background-color: "+color.name()+";";
		hintergrundfarbeLineEdit->setText(color.name());
		bcolorLineEdit->setStyleSheet(farbe);
		}
}

void BildershowGui::chooseTColor()
{
	QColor initialcolor;
	QString farbe;

	initialcolor.setNamedColor(titelfarbeLineEdit->text());

	QColor color = QColorDialog::getColor(initialcolor, this);
	if (color.isValid())
		{
		farbe="background-color: "+color.name()+";";
		titelfarbeLineEdit->setText(color.name());
		tcolorLineEdit->setStyleSheet(farbe);
		}
}

void BildershowGui::chooseLColor()
{
	QColor initialcolor;
	QString farbe;

	initialcolor.setNamedColor(linkfarbeLineEdit->text());

	QColor color = QColorDialog::getColor(initialcolor, this);
	if (color.isValid())
		{
		farbe="background-color: "+color.name()+";";
		linkfarbeLineEdit->setText(color.name());
		lcolorLineEdit->setStyleSheet(farbe);
		}
}

/***************************************************************
 Farbe anpassen bei Änderung der Zahl im jeweiligen QLineEdit
****************************************************************/

void BildershowGui::updateBColor()
{
	QColor color;
	color.setNamedColor(hintergrundfarbeLineEdit->text());
	if (color.isValid())
		bcolorLineEdit->setStyleSheet("background-color: "+color.name()+";");
}

void BildershowGui::updateTColor()
{
	QColor color;
	color.setNamedColor(titelfarbeLineEdit->text());
	if (color.isValid())
		tcolorLineEdit->setStyleSheet("background-color: "+color.name()+";");
}

void BildershowGui::updateLColor()
{
	QColor color;
	color.setNamedColor(linkfarbeLineEdit->text());
	if (color.isValid())
		lcolorLineEdit->setStyleSheet("background-color: "+color.name()+";");
}


/********************************************************
 thumbnails-SpinBox-Left bei Änderung in Right updaten
*********************************************************/

void BildershowGui::thumbssize(int thumbsRight)
{
	int valueLeft;
	double preValue;
	preValue = (double)thumbsRight*1.5;
	valueLeft = rundung(preValue);
	thumbsLeftSpinBox->setValue(valueLeft);
}

/***********************************************
 Test ob das Programm convert installiert ist
************************************************/

void BildershowGui::converttest()
{
	QFileInfo test("/usr/bin/convert");
	if(!test.exists())
		{
		font.setPointSize(12);
		QMessageBox msgBox;
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.setFont(font);
		msgBox.setIcon(QMessageBox::Critical);
		msgBox.setWindowTitle("Bildershow-gui");
		msgBox.setText(tr("<b>Fehler: convert nicht gefunden!</b><br><br>"
			"convert wird für die Erstellung der thumbnails benötigt.<br>"
			"Bitte installieren Sie convert aus dem<br>"
			"graphicsmagick-imagemagick-compat Paket.<br>"
			"Das Programm wird geschlossen."));
		msgBox.exec();
		QCoreApplication::exit(2);
		}
}

/********************************************************
 Fehlerfenster, falls Verzeichniserstellung fehlerhaft
*********************************************************/

void BildershowGui::errorDir()
{
		font.setPointSize(12);
		QMessageBox msgBox;
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.setFont(font);
		msgBox.setIcon(QMessageBox::Critical);
		msgBox.setWindowTitle("Bildershow-gui");
		msgBox.setText(tr("<b>Fehler beim erstellen des<br>"
			"Verzeichnisses oder der Dateien!</b><br>"
			"Fehlende Schreibrechte?<br><br>"
			"Dateien konnten nicht erstellt werden!"));
		msgBox.exec();
}


/********************************************************
 Erstellung der thumbnails mittels convert
*********************************************************/

void BildershowGui::procThumbs()
{
	QString Input = verzeichnis+picname[z];
	QString Output = verzThumbs+picname[z];
	QString tsize;
	tsize.setNum(hauflthumbs);
	tsize="x"+tsize;

	QStringList arg;
	arg << "-geometry" << tsize << "-sharpen" << "1" << "-quality" << "100" << Input << Output;

	proc.start("convert", arg);

}

void BildershowGui::convertBeendet(int exitCode, QProcess::ExitStatus exitStatus)
{
	if (exitStatus==QProcess::CrashExit || exitCode!=0)
		{
		openButton->setEnabled(true);
		createButton->setEnabled(true);
		QApplication::restoreOverrideCursor();
		errorDir();
		}
	else	{
		dateiTableWidget->setItem(z-1,2, new QTableWidgetItem("Fertig."));
		if(z<anz)
			{
			z++;
			procThumbs();
			}
		else if(z==anz)
			{
			openButton->setEnabled(true);
			createButton->setEnabled(true);
			QApplication::restoreOverrideCursor();
			}
		}
}

